# AI containers at the MPCDF

- preferred way to run AI applications at the MPCDF

## Quick start

```shell
module load image_pytorch  # i would prefer pytorch_sif
apptainer run $IMAGE_SIF example.py
```

- "module load" container
- run a python script with this container
- extend the base container
- run a slurm job with the extended container


## Intro to containers

- what are containers


### Apptainer

- basic apptainer commands


## Access the base containers

- `module load pytorch...`

- defines env variable $IMAGE_SIF
- maybe provide `kernel.json` spec files for jupyter lab?


## Build your own containers


> :warning: Do not build on the login nodes!!!


- Divide into two part:

### Our base images

- write your definition file

```shell
module load pytorch # or
echo $IMAGE_SIF
```

or

```shell
module show image_pytorch
module load apptainer/1.2.2
```

- copy paste

```definition file
...
```

```shell
apptainer build ...
```
### docker hub

 - nvidia containers for raven
 - amd container for viper

```
NVIDIA distribution (Raven): 

Tensorflow releases: https://docs.nvidia.com/deeplearning/frameworks/tensorflow-release-notes/index.html, published at https://catalog.ngc.nvidia.com/orgs/nvidia/containers/tensorflow
Pytorch: https://docs.nvidia.com/deeplearning/frameworks/pytorch-release-notes/index.html; published at https://catalog.ngc.nvidia.com/orgs/nvidia/containers/pytorch
AMD distribution (Viper): 

Compatibility matrix: https://rocm.docs.amd.com/projects/install-on-linux/en/latest/reference/docker-image-support-matrix.html
Pytorch: https://hub.docker.com/r/rocm/pytorch
TF: https://hub.docker.com/r/rocm/tensorflow**
```

### Bind stuff ...

- bind all your folder/data/volumes you need ...
- make user aware of possible python packages installed in his/her user directory


### Use your Python modules

Depending on the use case.
dev:
    - PYTHONPATH or
    - install in editable mode, always bind the package at the same location
    - warning when working with compiled files
static:
    - Copy files into the container


## Use containers in RVS

You can use the [IPython kernel](https://ipython.readthedocs.io/en/stable/install/kernel_install.html) in your container with a generic Jupyter Lab installation by providing a `kernel.json` spec file.

> :warning: Make sure the `apptainer` module is loaded when running a RVS session! 
> You can specify the default modules for your sessions during the initialization step.


### 1. Setting up the container

Make sure you install ipython and ipykernel in your container:
```
pip install ipython ipykernel
```

### 2. Setting up RVS

Load apptainer module when initializing your RVS session.

### 3. Creating the kernel

Create a kernel spec file
```bash
vim ~/.local/share/jupyter/kernels/my-kernel/kernel.json
```
that should look something like this
```json
{
 "argv": [
  "apptainer",
  "exec",
  "--nv",
  "--bind",
  "{connection_file}:/tmp/connection_spec,/ptmp/<your user name>",
  "/absolute/path/to/your/container.sif",
  "python",
  "-m",
  "ipykernel_launcher",
  "-f",
  "/tmp/connection_spec"
 ],
 "display_name": "Name of your kernel",
 "language": "python",
 "metadata": {
  "debugger": true
 },
 "env": {
  "PYTHONPATH": "/add/custom/modules/here"
 }
}
```

The next time you request a jupyter session, you can choose the generic jupyter version, and use your custom kernel.
Keep in mind that you are inside the container.
If you want to access files outside your home directory, you have to bind them explicitly in the kernel spec file when calling the apptainer command.
For example, in the kernel spec file above we bind your `ptmp` folder.

## Use containers in slurm jobs

- `module load apptainer`
- `srun apptainer ...`


### Distributed training with containers

- mention torchrun, accelerate launch


## Port containers: Local <-> MPCDF

- recommend docker for Windows/Macs
- Docker plugins available for VSCode / PyCharm; no idea how to use apptainer with these IDEs;


